OC.L10N.register(
    "sms_relentless",
    {
    "Settings" : "Settings",
    "SMS Relentless" : "SMS Relentless",
    "Please enter your credentials and options in the fields from below. As SMS service provider you can use Telnyx or Plivo or both." : "Please enter your credentials and options in the fields from below. As SMS service provider you can use Telnyx or Plivo or both.",
    "e.g. d6f172ce92e4f2 ..." : "e.g. d6f172ce92e4f2 ...",
    "e.g. KLCnMsTSp8utW3RRHx0FiXxbf5BxK2Y5q8mMTCn0jgv=" : "e.g. KLCnMsTSp8utW3RRHx0FiXxbf5BxK2Y5q8mMTCn0jgv=",
    "e.g. 9d4f47b6-26c9-d25c-c2f5 ..." : "e.g. 9d4f47b6-26c9-d25c-c2f5 ...",
    "Generate new webhook URL for incoming SMS" : "Generate new webhook URL for incoming SMS",
    "Generate new webhook URL for delivery receipts" : "Generate new webhook URL for delivery receipts",
    "e.g. Smart Inc" : "e.g. Smart Inc",
    "e.g. r5g974cb92e4t1 ..." : "e.g. r5g974cb92e4t1 ...",
    "e.g. k6f489d8awn4p9 ..." : "e.g. k6f489d8awn4p9 ...",
    "Generate new webhook URL for incoming SMS" : "Generate new webhook URL for incoming SMS",
    "Generate new webhook URL for delivery receipts" : "Generate new webhook URL for delivery receipts",
    "e.g. Global Inc" : "e.g. Global Inc",
    "e.g. 100" : "e.g. 100",
    "e.g. john.doe@example.com" : "e.g. john.doe@example.com",
    "Save" : "Save"
},
"nplurals=2; plural=(n != 1);");
